"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const FS = require("fs");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const devKitCore = require("@angular-devkit/core");
const architect_1 = require("@angular-devkit/architect");
const build_ng_packagr_1 = require("@angular-devkit/build-ng-packagr");
const ngPackagr = require("ng-packagr");
const create_hook_provider_1 = require("./create-hook-provider");
const utils_1 = require("./utils");
const hook_registry_1 = require("./hook-registry");
__export(require("./hook-registry"));
var job_1 = require("./job");
exports.Job = job_1.Job;
var utils_2 = require("./utils");
exports.ENTRY_POINT_STORAGE = utils_2.ENTRY_POINT_STORAGE;
const DEFAULT_TSCONFIG_OPTIONS = {
    moduleResolution: 'node',
    module: 'commonjs',
    target: 'es6',
    lib: [
        'es2017',
        'dom'
    ],
};
function buildRegistry(globalTasksContext) {
    return __awaiter(this, void 0, void 0, function* () {
        const root = globalTasksContext.root;
        const { tasks } = globalTasksContext.options;
        const transformerPath = tasks.config;
        const tPath = devKitCore.getSystemPath(devKitCore.resolve(root, devKitCore.normalize(transformerPath)));
        if (FS.existsSync(tPath)) {
            if (/\.ts$/.test(tPath) && !require.extensions['.ts']) {
                const tsNodeOptions = {};
                if (tasks.tsConfig) {
                    tsNodeOptions.project = tasks.tsConfig;
                }
                else {
                    tsNodeOptions.compilerOptions = DEFAULT_TSCONFIG_OPTIONS;
                }
                require('ts-node').register(tsNodeOptions);
            }
            const transformHooksModule = require(tPath);
            if (typeof transformHooksModule === 'function') {
                const registry = new hook_registry_1.HookRegistry();
                yield transformHooksModule(globalTasksContext, registry);
                return registry;
            }
            else {
                const registry = new hook_registry_1.HookRegistry(transformHooksModule);
                return registry;
            }
        }
        ;
        return Promise.resolve(new hook_registry_1.HookRegistry());
    });
}
function initRegistry(options, builderContext) {
    return __awaiter(this, void 0, void 0, function* () {
        const context = yield utils_1.createHooksContext(options, builderContext);
        const registry = yield buildRegistry(context);
        return { context, registry };
    });
}
function execute(options, context) {
    const { build, watch } = ngPackagr.NgPackagr.prototype;
    return rxjs_1.from(initRegistry(options, context))
        .pipe(operators_1.switchMap(result => utils_1.validateTypedTasks(result.registry.getJobs(), result.context).then(() => result)), operators_1.tap(result => {
        const providers = create_hook_provider_1.createHookProviders(result.registry.getHooks(), result.context);
        ngPackagr.NgPackagr.prototype.build = function () {
            this.withProviders(providers);
            return build.call(this);
        };
        ngPackagr.NgPackagr.prototype.watch = function () {
            this.withProviders(providers);
            if (result.registry.hasSelfWatchJob) {
                return this.buildAsObservable();
            }
            else {
                return watch.call(this);
            }
        };
    }), operators_1.switchMap(() => build_ng_packagr_1.execute(options, context)), operators_1.tap(buildEvent => {
        ngPackagr.NgPackagr.prototype.build = build;
        ngPackagr.NgPackagr.prototype.watch = watch;
    }));
}
exports.execute = execute;
exports.default = architect_1.createBuilder(execute);
//# sourceMappingURL=index.js.map